import os
import argparse

from mommys_boy import __version__


def get_parser():
    """
    Creates a new argument parser.
    """
    parser = argparse.ArgumentParser('Mommy's Boy')
    version = '%(prog)s ' + __version__
    parser.add_argument('--version', '-v', action='version', version=version)
    return parser


def main(args=None):
    """
    Called with ``python -m mommys_boy.tests``: run main test suite.
    """

    parser = get_parser()
    args = parser.parse_args(args)

    # Check if pytest is available
    try:
        import pytest
    except ImportError:
        raise SystemExit(
            'You need py.test to run the test suite.\n'
            'You can install it using your distribution package manager or\n'
            '    $ python -m pip install pytest --user'
        )

    # Get data from test_module
    import mommys_boy.tests as test_module
    test_path = os.path.abspath(os.path.dirname(test_module.__file__))
    pytest.main([test_path, '-m', 'not documentation'])


if __name__ == '__main__':
    main()


from __future__ import absolute_import
from __future__ import unicode_literals

import warnings

from django.db.models import Model as Model
from factory import Factory
from factory.django import DjangoModelFactory, DjangoOptions
from factory import base
from faker import Factory as FakeFactory

fake = FakeFactory.create()


class MommyOptions(DjangoOptions):
    """
    Define additional options for DjangoMommyFactory subclasses.
    """

    def _build_default_options(self):
        return super()._build_default_options() + [
            base.OptionDefault('auto_fields', None, inherit=True),
            base.OptionDefault('auto_exclude', (), inherit=True),
            base.OptionDefault('recipe', None, inherit=True),
        ]


class MommyMeta(type(DjangoModelFactory)):
    """
    Metaclass for DjangoMommyFactory models.
    """
    _global_models_cache = {}
    _forbidden_fields = ('id',)
    _fake_extra = {
        name.replace('_', ''): getattr(fake, name)
        for name in dir(fake)
        if not name.startswith('_')
        }

    def __init__(self, *args):
        super().__init__(*args)

        self._forbidden_fields += self._get_declared_fields()
        self._forbidden_fields += self._meta.auto_exclude
        if self._meta.recipe == 'global':
            model = self._meta.model
            if model not in self._global_models_cache:
                self._global_models_cache[model] = self
            else:
                warnings.warn(
                    'global DjangoMommyFactory for %s.%s is already '
                    'registered. ignoring it.' % (model.__module__,
                                                  model.__name__),
                    RuntimeWarning,
                    stacklevel=2,
                )

        if self._meta.abstract is False:
            self._faker_fields = self._get_faker_fields()

    def _get_faker_fields(self):
        """
        Return a mapping of name: faker_function for all fields that have a name
        corresponding to a faker function.

        Fields in the ._forbidden_fields list are not included.
        """

        fields = {}
        for field in self._meta.model._meta.fields:
            name = field.name
            if (name in self._forbidden_fields or
                    (self._meta.auto_fields is not None and
                     name not in self._meta.auto_fields)):
                continue

            # Try to find a fake function for the given field
            faker = getattr(fake, name, None)
            if faker is None:
                faker = getattr(fake, name.replace('_', ''), None)
            if faker is None:
                faker = self._fake_extra.get(name, None)
            if faker is None:
                continue

            fields[name] = faker

        return fields

    def _get_declared_fields(self):
        """
        Return a tuple with all explicit field declarations.
        """

        return tuple(self._meta.declarations)


class MommyFactoryBase(object):
    class Meta:
        abstract = True

    _options_class = MommyOptions

    @classmethod
    def _create(cls, model_class, *args, **kwargs):
        return cls._mommy_build_or_create(True, model_class, *args, **kwargs)

    @classmethod
    def _build(cls, model_class, *args, **kwargs):
        return cls._mommy_build_or_create(False, model_class, *args, **kwargs)

    @classmethod
    def _mommy_build_or_create(cls, create, model_class, *args, **kwargs):
        from model_mommy import mommy
        kwargs = cls._normalize_args(*args, **kwargs)
        if create:
            return mommy.make(model_class, **kwargs)
        else:
            return mommy.prepare(model_class, **kwargs)

    @classmethod
    def _normalize_args(cls, *args, **kwargs):
        kwargs = dict(cls._faker_fields, **kwargs)
        if args:
            raise NotImplemented('positional arguments are not supported')
        return kwargs

    @classmethod
    def _get_mommy_args(cls, _create, **kwargs):
        return cls._normalize_args(**cls.attributes(_create, kwargs))


class MommyManager(object):
    """
    Implements the global mommy object.
    """

    @property
    def _mommy(self):
        try:
            return self.__mommy
        except AttributeError:
            from model_mommy import mommy
            self.__mommy = mommy
            return mommy

    def __init__(self):
        self._model_cache = {}

    def get_factory(self, model_class):
        """
        Return the DjangoMommyFactory subclass associated with the given model.
        """
        if isinstance(model_class, str):
            model_class = self._mommy.Mommy.finder.get_model(model_class)

        factory = MommyMeta._global_models_cache.get(model_class, None)
        if factory is None:
            factory = self._model_cache.get(model_class, None)
        if factory is None:
            class Factory(DjangoMommyFactory):
                class Meta:
                    model = model_class

            factory = self._model_cache[model_class] = Factory
        return factory

    def _get_mommy_args(self, model, _create, **kwargs):
        if issubclass(model, Model):
            factory = self.get_factory(model)
        elif issubclass(model, DjangoMommyFactory):
            factory = model
        elif issubclass(model, Factory):
            return Factory.attributes(_create, kwargs)
        else:
            raise TypeError('argument must be a model instance or a '
                            'DjangoMommyFactory subclass')
        kwargs = factory._get_mommy_args(_create, **kwargs)
        return kwargs

    def _get_model(self, model):
        if issubclass(model, Model):
            return model
        return model._meta.model

    def make(self, model, _quantity=None, make_m2m=False, **kwargs):
        kwargs = self._get_mommy_args(model, True, **kwargs)
        model = self._get_model(model)
        return self._mommy.make(model, _quantity, make_m2m, **kwargs)

    def prepare(self, model, _quantity=None, **kwargs):
        kwargs = self._get_mommy_args(model, False, **kwargs)
        model = self._get_model(model)
        return self._mommy.prepare(model, _quantity, **kwargs)

    def create(self, *args, **kwargs):
        """
        An alias to mommy.make()
        """

        return self.make(*args, **kwargs)

    def build(self, *args, **kwargs):
        """
        An alias to mommy.prepare()
        """

        return self.prepare(*args, **kwargs)

# Initialize export symbols
DjangoMommyFactory = MommyMeta(
    'DjangoMommyFactory',
    (MommyFactoryBase, DjangoModelFactory), {
        '__doc__': """
        Base class for Mommy-enabled factories.
        """
    }
)
mommy = MommyManager()



"""
Django settings for testsite project.

Generated by 'django-admin startproject' using Django 1.9.6.

For more information on this file, see
https://docs.djangoproject.com/en/1.9/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.9/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.9/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '@8o_o#!2x!67*7&li1&rq0%05q1l80^7bj0ww0qdiuz@4pg-h6'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'mommys_boy.tests.app',
]

MIDDLEWARE_CLASSES = [
    'django.middleware.common.CommonMiddleware',
]

ROOT_URLCONF = 'mommys_boy.tests.site.urls'

TEMPLATES = []

WSGI_APPLICATION = 'mommys_boy.tests.site.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.9/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
        'TEST': {
            'NAME': os.path.join(BASE_DIR, 'testdb.sqlite3'),
        }
    }
}


# Password validation
# https://docs.djangoproject.com/en/1.9/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = []


# Internationalization
# https://docs.djangoproject.com/en/1.9/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = False

USE_L10N = False

USE_TZ = False



import pytest
from django.contrib.auth.models import User
from mommys_boy.tests.app.models import TestModel, TestFactoryUnix
from mommys_boy import mommy, DjangoMommyFactory, DjangoModelFactory


@pytest.fixture
def user_factory():
    class UserFactory(DjangoMommyFactory):
        class Meta:
            model = User
    return UserFactory


@pytest.fixture
def user_std_factory():
    class UserFactory(DjangoModelFactory):
        class Meta:
            model = User
    return UserFactory


# Common implementation for mommy.make, mommy.prepare testers
def _test_user(*users):
    for user in users:
        assert user.username
        assert user.password
        assert user.email
        assert user.first_name
        assert user.last_name


#
# Mommy tests
#
@pytest.mark.django_db
def test_mommy_make():
    _test_user(mommy.make(User))


def test_mommy_prepare():
    _test_user(mommy.prepare(User))


def test_mommy_prepare_aliases():
    u1 = mommy.prepare(User)
    u2 = mommy.build(User)
    _test_user(u1, u2)
    assert u1.__dict__.keys() == u2.__dict__.keys()


@pytest.mark.django_db
def test_mommy_aliases():
    u1 = mommy.make(User)
    u2 = mommy.create(User)
    assert u1.__dict__.keys() == u2.__dict__.keys()


def test_mommy_accepts_mommy_factories(user_factory):
    user = mommy.prepare(user_factory)
    _test_user(user)


def test_mommy_accepts_std_factories(user_std_factory):
    user = mommy.prepare(user_std_factory)
    assert user.username
    assert user.password
    assert not user.email
    assert not user.first_name
    assert not user.last_name


#
# Factory tests
#
def test_factory_build(user_factory):
    user = user_factory.build()
    _test_user(user)


@pytest.mark.django_db
def test_factory_create(user_factory):
    user = user_factory.create()
    _test_user(user)


def test_global_recipe_factory():
    obj = mommy.build(TestModel)
    assert obj.some_number == 42
    assert obj.some_date
    obj.full_clean()


def test_recipe_factory():
    obj = mommy.build(TestFactoryUnix)
    assert obj.some_date.year == 1970
    assert obj.some_date.month == 1
    assert obj.some_date.day == 1
    assert obj.some_number
    obj.full_clean()



from datetime import date
from django.db import models
from mommys_boy import DjangoMommyFactory


class TestModel(models.Model):
    first_name = models.CharField(max_length=50)
    some_number = models.IntegerField()
    some_date = models.DateField()


class TestFactory(DjangoMommyFactory):
    class Meta:
        model = TestModel
        recipe = 'global'

    some_number = 42


class TestFactoryUnix(DjangoMommyFactory):
    class Meta:
        model = TestModel
        recipe = 'unix-time'

    some_date = date(1970, 1, 1)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-29 13:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=50)),
                ('some_number', models.IntegerField()),
                ('some_date', models.DateField()),
            ],
        ),
    ]
